void drawData()
{
  if (rotate) 
    rotateY(PI/2);

  if (flower)
  {
    rotateX(7*PI/12);
    pushMatrix();
    for (int i =0; i<10; i++)
    {
      rotateY(TWO_PI/9);
      drawDewey(i);
    }
    popMatrix();
  }

  for (int i =0; i<10; i++)
  {
    if (state[i]) {
      drawDewey(i);
      if (dot)
        drawDot(i);
    }
  }

  if (notation)
    drawNotation();
}


////// show different dewey sections
void drawDewey(int d) {

  stroke(clr[d], 100);

  for (int i=lowS; i<highS; i++)
  {
    for (int j=lowY; j<highY; j++)
    {
      int z = (int)map(j*15+i, 0, 150, -len, len);

      strokeWeight(1);
      beginShape();     
      curveVertex((radius+co*data[d][i][j][0])*cos(-0.5*PI), (radius+co*data[d][i][j][0])*sin(-0.5*PI), z);
      for (int m=0; m<10; m++)
      {
        curveVertex((radius+co*data[d][i][j][m])*cos(-0.5*PI+2*m*PI/10), (radius+co*data[d][i][j][m])*sin(-0.5*PI+2*m*PI/10), z);
      }
      curveVertex((radius+co*data[d][i][j][0])*cos(-0.5*PI), (radius+co*data[d][i][j][0])*sin(-0.5*PI), z);
      curveVertex((radius+co*data[d][i][j][1])*cos(-0.3*PI), (radius+co*data[d][i][j][1])*sin(-0.3*PI), z);
      endShape();
    }
  }
}


////  labels with dots, and details 
void drawDot(int d)
{
  stroke(clr[d], 100);

  for (int i=lowS; i<highS; i++)
  {
    for (int j=lowY; j<highY; j++)
    {
      int z = (int)map(j*15+i, 0, 150, -len, len);
      for (int m=0; m<10; m++)
      {
        pushMatrix();
        
        float[] rotations = cam.getRotations();
        strokeWeight(3);
        translate((radius+co*data[d][i][j][m])*cos(-0.5*PI+2*m*PI/10), (radius+co*data[d][i][j][m])*sin(-0.5*PI+2*m*PI/10), z);   
        rotateX(rotations[0]);   //// make text always face the front
        rotateY(rotations[1]);
        rotateZ(rotations[2]);
        point(0, 0, 0);
        float mouseObjectDistance = sq(mouseX-screenX(0, 0, 0))+sq(mouseY-screenY(0, 0, 0)); 
        if (mouseObjectDistance < 20)
        {
          stroke(250);
          textMode(MODEL);
          textSize(10);
          text("Dewey class: "+(d*100+i*10+m), 5, 0);
          text("Year: "+ (j+2006), 5, 12);
          text("Average title length: "+data[d][i][j][m], 5, 22);
          stroke(clr[d], 100);
          a = d;
          b = i;
          c = j;
        }
        dist=(int)mouseObjectDistance;
        popMatrix();
        if (dist<20)
          hightlight(a, b, c);
      }
    }
  }
}


/////////  highlight the chosen curve /////////////////////
void hightlight(int d, int i, int j) {

  int z = (int)map(j*15+i, 0, 150, -len, len);
  strokeWeight(5);
  beginShape();     
  curveVertex((radius+co*data[d][i][j][0])*cos(-0.5*PI), (radius+co*data[d][i][j][0])*sin(-0.5*PI), z);
  for (int m=0; m<10; m++)
  {
    curveVertex((radius+co*data[d][i][j][m])*cos(-0.5*PI+2*m*PI/10), (radius+co*data[d][i][j][m])*sin(-0.5*PI+2*m*PI/10), z);
  }
  curveVertex((radius+co*data[d][i][j][0])*cos(-0.5*PI), (radius+co*data[d][i][j][0])*sin(-0.5*PI), z);
  curveVertex((radius+co*data[d][i][j][1])*cos(-0.3*PI), (radius+co*data[d][i][j][1])*sin(-0.3*PI), z);
  endShape();
}


/////////////////   basic information //////////////////
void drawInfo() {
  //fill(70);
  //noStroke();
  //rect(0,0,width/3, height/3);


///// title ////////////
  strokeWeight(0.5);
  stroke(fontColor[0]);
  textSize(fontSize[0]);
  textAlign(LEFT, CENTER);
  text("TIME or TIMELESS?", 20, 40);
  line(20, 55, 270, 55);

  //subheading
  stroke(fontColor[1]);
  textSize(fontSize[2]);
  text("Average Title Length in Different Dewey Classes", 20, 65);
  text("Data Source: Seattle Public Library", 20, 80);
  textSize(fontSize[1]);
  text("MAT 259 | JINGYI XIAO", 20, 110);

  noFill();
}


////////// draw the notation /////////////////////////
void drawNotation() {

  ///draw the line 
  strokeWeight(2);
  stroke(250);
  line(0, 0, -len-300, 0, 0, len+300);

  /// draw the time circle from 2006 to 2016
  strokeWeight(1);
  stroke(250, 50); 
  for (int i=0; i<11; i++)
  {
    pushMatrix();
    translate(0, 0, i*2*len/10-len);
    ellipse(0, 0, 6*radius, 6*radius);
    textSize(32);
    text(2006+i, 3*radius+10, 0);
    popMatrix();
  }

  //// draw the sector
  textSize(24);
  for (int i=0; i<11; i++)
  {
    pushMatrix();
    translate(0, 0, i*2*len/10-len);
    pushMatrix(); 
    line(0, 3*radius-10, 0, 0, 3*radius, 0);
    text(0, 0, 30-3*radius);
    for (int j=1; j<10; j++)
    {
      rotateZ(PI/5);
      line(0, 3*radius-10, 0, 0, 3*radius, 0);
      text(j, 0, 30-3*radius);
    }
    popMatrix();
    popMatrix();
  }
}